/*
SDLPoP, a port/conversion of the DOS game Prince of Persia.
Copyright (C) 2013-2017  Dávid Nagy

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

The authors of this program may be contacted at http://forum.princed.org
*/

#include "common.h"

#define TABS 7
#define MAX_PATHFILE 500
#define MAX_NUMBER 4
/***/
#define MINUTES_LEFT "Minutes left:"
#define HITPOINTS_LEFT "Hitpoints left:"
#define GAMEPAD_RUMBLE "Gamepad rumble:"
#define LIGHTING "Enable lighting:"

/*****************************************************************************/
int SettingsLoop (void *unused)
/*****************************************************************************/
{
	/* This function is currently not in use. In theory, this could be
	 * called after SettingsPreLoad(); in set_gr_mode(), as:
	 * SDL_CreateThread (SettingsLoop, "SettingsLoop", NULL);
	 */

	Uint32 looptime;

	while (1)
	{
		looptime = SDL_GetTicks();
		SettingsShow();
		SDL_RenderPresent (renderer2_);
		while ((SDL_GetTicks() - looptime) < 25) { SDL_Delay (10); }
	}

	return (0);
}
/*****************************************************************************/
void SettingsButtonDown (SDL_Event event)
/*****************************************************************************/
{
	int iVisible;

	/*** Used for looping. ***/
	int iLoopTabs;

	if (event.button.button == 1)
	{
		if (event.button.windowID == window_ID)
		{
			iVisible = (SDL_GetWindowFlags (window2_) & SDL_WINDOW_SHOWN);
			if (iVisible == 0)
			{
				SDL_ShowWindow (window2_);
			} else { /*** 4 ***/
				SDL_HideWindow (window2_);
			}
/***
TODO:
For some reason, after re-hiding, TWO clicks are required to show the window.
The same happens with the code below.

uint32_t flags = SDL_GetWindowFlags (window2_);
if (flags & SDL_WINDOW_SHOWN)
{
	SDL_HideWindow (window2_);
} else {
	SDL_ShowWindow (window2_);
}
***/
		}
		if (event.button.windowID == window2_ID)
		{
			for (iLoopTabs = 1; iLoopTabs <= TABS; iLoopTabs++)
			{
				if (InArea (15, -11 + (29 * iLoopTabs), 94, 30) == 1)
					{ iTabActive = iLoopTabs; }
			}

			if (InArea (17, 304, 75, 32) == 1) { iButtonDown = 1; } /*** Save ***/
			if (InArea (17, 351, 75, 32) == 1) { iButtonDown = 2; } /*** Close ***/

			if (iTabActive == 1)
			{
				/*** MINUTES_LEFT ***/
				if (ClickedValue (1, 1) == 1) { start_minutes_left++; }
				if (ClickedValue (1, 2) == 1) { start_minutes_left--; }
				if (ClickedValue (1, 3) == 1) { rem_min++; }
				if (ClickedValue (1, 4) == 1) { rem_min--; }

				/*** HITPOINTS_LEFT ***/
				if (ClickedValue (2, 1) == 1) { start_hitp++; }
				if (ClickedValue (2, 2) == 1) { start_hitp--; }
				if (ClickedValue (2, 3) == 1) { hitp_curr++; }
				if (ClickedValue (2, 4) == 1) { hitp_curr--; }

				/*** GAMEPAD_RUMBLE ***/
				/*** TODO: Same here [2/3]. ***/
				if (ClickedByte (3, 1) == 1) { Toggle (&enable_controller_rumble); }
				if (ClickedByte (3, 2) == 1) { Toggle (&enable_controller_rumble); }

				/*** LIGHTING ***/
				/*** TODO: And here [3/3]. ***/
				if (ClickedByte (4, 1) == 1) { Toggle (&enable_lighting); }
				if (ClickedByte (4, 2) == 1) { Toggle (&enable_lighting); }
			}
		}
	}
}
/*****************************************************************************/
void SettingsButtonUp (SDL_Event event)
/*****************************************************************************/
{
	char sPathFile[MAX_PATHFILE + 2];

	iButtonDown = 0;
	if (event.button.button == 1)
	{
		if (event.button.windowID == window2_ID)
		{
			/*** Save ***/
			if (InArea (17, 304, 75, 32) == 1)
			{
				if (use_custom_levelset == 0)
				{
					snprintf (sPathFile, MAX_PATHFILE, "%s", "SDLPoP.ini");
				} else {
					snprintf (sPathFile, MAX_PATHFILE, "mods/%s/%s",
						levelset_name, "mod.ini");
				}
				printf ("TODO: Save all settings to \"%s\".\n", sPathFile);
			}

			/*** Close ***/
			if (InArea (17, 351, 75, 32) == 1)
			{
				SDL_HideWindow (window2_);
			}
		}
	}
}
/*****************************************************************************/
int InArea (int iUpperLeftX, int iUpperLeftY, int iWidth, int iHeight)
/*****************************************************************************/
{
	if ((iUpperLeftX <= iXPos) &&
		(iUpperLeftX + iWidth >= iXPos) &&
		(iUpperLeftY <= iYPos) &&
		(iUpperLeftY + iHeight >= iYPos))
	{
		return (1);
	} else {
		return (0);
	}
}
/*****************************************************************************/
void SettingsPreLoad (void)
/*****************************************************************************/
{
	char sPathFile[MAX_PATHFILE + 2];

	/*** Used for looping. ***/
	int iLoopLetter;
	int iLoopDigit;

	PreLoad ("data/settings/settings.png", &imgsettings);
	PreLoad ("data/settings/tab.png", &imgtab);
	PreLoad ("data/settings/tab_active.png", &imgtab_active);
	PreLoad ("data/settings/number.png", &imgnumber);
	PreLoad ("data/settings/button_up.png", &imgbutton_up);
	PreLoad ("data/settings/button_down.png", &imgbutton_down);
	PreLoad ("data/settings/checkbox.png", &imgcheckbox);

	/*** font ***/
	for (iLoopLetter = 1; iLoopLetter <= 26; iLoopLetter++)
	{
		snprintf (sPathFile, MAX_PATHFILE,
			"data/settings/font/%c.png", iLoopLetter + 96);
		PreLoad (sPathFile, &imglower[iLoopLetter]);
		snprintf (sPathFile, MAX_PATHFILE,
			"data/settings/font/%c.png", iLoopLetter + 64);
		PreLoad (sPathFile, &imgupper[iLoopLetter]);
	}
	for (iLoopDigit = 0; iLoopDigit <= 9; iLoopDigit++)
	{
		snprintf (sPathFile, MAX_PATHFILE,
			"data/settings/font/%c.png", iLoopDigit + 48);
		PreLoad (sPathFile, &imgdigit[iLoopDigit]);
	}
	PreLoad ("data/settings/font/ALT.png",
		&imgALT); /*** ALT ***/
	PreLoad ("data/settings/font/ampersand.png",
		&imgampersand); /*** '&' ***/
	PreLoad ("data/settings/font/apostrophe.png",
		&imgapostrophe); /*** ''' ***/
	PreLoad ("data/settings/font/asterisk.png",
		&imgasterisk); /*** '*' ***/
	PreLoad ("data/settings/font/at_sign.png",
		&imgat_sign); /*** '@' ***/
	PreLoad ("data/settings/font/backslash.png",
		&imgbackslash); /*** '\' ***/
	PreLoad ("data/settings/font/braces_left.png",
		&imgbraces_left); /*** '{' ***/
	PreLoad ("data/settings/font/braces_right.png",
		&imgbraces_right); /*** '}' ***/
	PreLoad ("data/settings/font/bracket_left.png",
		&imgbracket_left); /*** '[' ***/
	PreLoad ("data/settings/font/bracket_right.png",
		&imgbracket_right); /*** ']' ***/
	PreLoad ("data/settings/font/vertical_bar.png",
		&imgvertical_bar); /*** '|' ***/
	PreLoad ("data/settings/font/caret.png",
		&imgcaret); /*** '^' ***/
	PreLoad ("data/settings/font/check_mark.png",
		&imgcheck_mark); /*** check mark ***/
	PreLoad ("data/settings/font/colon.png",
		&imgcolon); /*** ':' ***/
	PreLoad ("data/settings/font/comma.png",
		&imgcomma); /*** ',' ***/
	PreLoad ("data/settings/font/CTRL.png",
		&imgCTRL); /*** CTRL ***/
	PreLoad ("data/settings/font/dollar_sign.png",
		&imgdollar_sign); /*** '$' ***/
	PreLoad ("data/settings/font/equals_sign.png",
		&imgequals_sign); /*** '=' ***/
	PreLoad ("data/settings/font/ESC.png",
		&imgESC); /*** ESC ***/
	PreLoad ("data/settings/font/exclamation_mark.png",
		&imgexclamation_mark); /*** '!' ***/
	PreLoad ("data/settings/font/grave_accent.png",
		&imggrave_accent); /*** '`' ***/
	PreLoad ("data/settings/font/guillemet_left.png",
		&imgguillemet_left); /*** '<' ***/
	PreLoad ("data/settings/font/guillemet_right.png",
		&imgguillemet_right); /*** '>' ***/
	PreLoad ("data/settings/font/hyphen_minus.png",
		&imghyphen_minus); /*** '-' ***/
	PreLoad ("data/settings/font/number_sign.png",
		&imgnumber_sign); /*** '#' ***/
	PreLoad ("data/settings/font/parentheses_close.png",
		&imgparentheses_close); /*** ')' ***/
	PreLoad ("data/settings/font/parentheses_open.png",
		&imgparentheses_open); /*** '(' ***/
	PreLoad ("data/settings/font/percent_sign.png",
		&imgpercent_sign); /*** '%' ***/
	PreLoad ("data/settings/font/period.png",
		&imgperiod); /*** '.' ***/
	PreLoad ("data/settings/font/plus_sign.png",
		&imgplus_sign); /*** '+' ***/
	PreLoad ("data/settings/font/question_mark.png",
		&imgquestion_mark); /*** '?' ***/
	PreLoad ("data/settings/font/quotation_marks.png",
		&imgquotation_marks); /*** '"' ***/
	PreLoad ("data/settings/font/semicolon.png",
		&imgsemicolon); /*** ';' ***/
	PreLoad ("data/settings/font/slashed_o.png",
		&imgslashed_o); /*** slashed o ***/
	PreLoad ("data/settings/font/slash.png",
		&imgslash); /*** '/' ***/
	PreLoad ("data/settings/font/space.png",
		&imgspace); /*** ' ' ***/
	PreLoad ("data/settings/font/tilde.png",
		&imgtilde); /*** '~' ***/
	PreLoad ("data/settings/font/underscore.png",
		&imgunderscore); /*** '_' ***/
}
/*****************************************************************************/
void PreLoad (char *sPathFile, SDL_Texture **imgImage)
/*****************************************************************************/
{
	*imgImage = IMG_LoadTexture (renderer2_, sPathFile);
	if (!*imgImage)
	{
		printf ("[FAILED] IMG_LoadTexture: %s!\n", IMG_GetError());
		exit (-1);
	}
}
/*****************************************************************************/
int ShowImage (SDL_Texture *img, int iX, int iY, char *sImageInfo)
/*****************************************************************************/
{
	SDL_Rect loc, dest;
	int iWidth, iHeight;

	SDL_QueryTexture (img, NULL, NULL, &iWidth, &iHeight);
	loc.x = 0; loc.y = 0; loc.w = iWidth; loc.h = iHeight;
	dest.x = iX; dest.y = iY; dest.w = iWidth; dest.h = iHeight;
	CustomRenderCopy (img, &loc, &dest, sImageInfo);

	return (iWidth);
}
/*****************************************************************************/
void CustomRenderCopy (SDL_Texture* src, SDL_Rect* srcrect,
	SDL_Rect *dstrect, char *sImageInfo)
/*****************************************************************************/
{
	if (SDL_RenderCopy (renderer2_, src, srcrect, dstrect) != 0)
	{
		printf ("[ WARN ] SDL_RenderCopy (%s): %s!\n",
			sImageInfo, SDL_GetError());
	}
}
/*****************************************************************************/
void SettingsShow (void)
/*****************************************************************************/
{
	/*** Used for looping. ***/
	int iLoopTabs;

	ShowImage (imgsettings, 0, 0, "imgsettings");
	for (iLoopTabs = 1; iLoopTabs <= TABS; iLoopTabs++)
	{
		if (iLoopTabs == iTabActive)
		{
			ShowImage (imgtab_active, 15, -11 + (29 * iLoopTabs), "imgtab_active");
		} else {
			ShowImage (imgtab, 15, -11 + (29 * iLoopTabs), "imgtab");
		}
	}

	ShowText ("INI", 431, 36);
	ShowText ("live", 526, 36);

	switch (iTabActive)
	{
		case 1:
			LineValue (MINUTES_LEFT, 1,
				start_minutes_left, rem_min);
			LineValue (HITPOINTS_LEFT, 2,
				start_hitp, hitp_curr);
			/*** TODO: Game should split these [1/3]. ***/
			LineByte (GAMEPAD_RUMBLE, 3,
				enable_controller_rumble, enable_controller_rumble);
			LineByte (LIGHTING, 4, enable_lighting, enable_lighting);
			break;
	}

	/*** Save ***/
	if (iButtonDown == 1)
	{
		ShowImage (imgbutton_down, 17, 304, "imgbutton_down");
		ShowText ("Save", 26 + 1, 313 + 1);
	} else {
		ShowImage (imgbutton_up, 17, 304, "imgbutton_up");
		ShowText ("Save", 26, 313);
	}

	/*** Close ***/
	if (iButtonDown == 2)
	{
		ShowImage (imgbutton_down, 17, 351, "imgbutton_down");
		ShowText ("Close", 26 + 1, 360 + 1);
	} else {
		ShowImage (imgbutton_up, 17, 351, "imgbutton_up");
		ShowText ("Close", 26, 360);
	}
}
/*****************************************************************************/
void LineValue (char *sLabel, int iRow, int iINI, int iLive)
/*****************************************************************************/
{
	int iY;
	char sNumber[MAX_NUMBER + 2];

	iY = 36 + (iRow * 33);
	ShowText (sLabel, 126, iY);
	ShowImage (imgnumber, 431, iY - 2, "imgnumber");
	snprintf (sNumber, MAX_NUMBER, "%i", iINI);
	ShowText (sNumber, 431 + 22, iY);
	ShowImage (imgnumber, 526, iY - 2, "imgnumber");
	snprintf (sNumber, MAX_NUMBER, "%i", iLive);
	ShowText (sNumber, 526 + 22, iY);
}
/*****************************************************************************/
void LineByte (char *sLabel, int iRow, byte bINI, byte bLive)
/*****************************************************************************/
{
	int iY;

	iY = 36 + (iRow * 33);
	ShowText (sLabel, 126, iY);
	ShowImage (imgcheckbox, 431, iY - 2, "imgcheckbox");
	if (bINI == 1)
		{ ShowImage (imgcheck_mark, 431 + 2, iY - 1, "imgcheck_mark"); }
	ShowImage (imgcheckbox, 526, iY - 2, "imgcheckbox");
	if (bLive == 1)
		{ ShowImage (imgcheck_mark, 526 + 2, iY - 1, "imgcheck_mark"); }
}
/*****************************************************************************/
void ShowText (char *sText, int iX, int iY)
/*****************************************************************************/
{
	int iXStart, iWidth;

	/*** Used for looping. ***/
	int iLoopChar;

	iXStart = iX;
	for (iLoopChar = 0; iLoopChar < strlen (sText); iLoopChar++)
	{
		iWidth = 6; /*** Fallback. ***/
		/*** a-z ***/
		if ((sText[iLoopChar] >= 97) && (sText[iLoopChar] <= 122))
		{
			iWidth = ShowImage (imglower[sText[iLoopChar] - 96],
				iXStart, iY, "imglower[]");
		}
		/*** A-Z ***/
		if ((sText[iLoopChar] >= 65) && (sText[iLoopChar] <= 90))
		{
			iWidth = ShowImage (imgupper[sText[iLoopChar] - 64],
				iXStart, iY, "imgupper[]");
		}
		/*** 0-9 ***/
		if ((sText[iLoopChar] >= 48) && (sText[iLoopChar] <= 57))
		{
			iWidth = ShowImage (imgdigit[sText[iLoopChar] - 48],
				iXStart, iY, "imgdigit[]");
		}
		switch (sText[iLoopChar])
		{
			/*** ALT ***/
			case '&': iWidth = ShowImage
				(imgampersand, iXStart, iY, "imgampersand"); break;
			case '\'': iWidth = ShowImage
				(imgapostrophe, iXStart, iY, "imgapostrophe"); break;
			case '*': iWidth = ShowImage
				(imgasterisk, iXStart, iY, "imgasterisk"); break;
			case '@': iWidth = ShowImage
				(imgat_sign, iXStart, iY, "imgat_sign"); break;
			case '\\': iWidth = ShowImage
				(imgbackslash, iXStart, iY, "imgbackslash"); break;
			case '{': iWidth = ShowImage
				(imgbraces_left, iXStart, iY, "imgbraces_left"); break;
			case '}': iWidth = ShowImage
				(imgbraces_right, iXStart, iY, "imgbraces_right"); break;
			case '[': iWidth = ShowImage
				(imgbracket_left, iXStart, iY, "imgbracket_left"); break;
			case ']': iWidth = ShowImage
				(imgbracket_right, iXStart, iY, "imgbracket_right"); break;
			case '|': iWidth = ShowImage
				(imgvertical_bar, iXStart, iY, "imgvertical_bar"); break;
			case '^': iWidth = ShowImage
				(imgcaret, iXStart, iY, "imgcaret"); break;
			/*** check_mark ***/
			case ':': iWidth = ShowImage
				(imgcolon, iXStart, iY, "imgcolon"); break;
			case ',': iWidth = ShowImage
				(imgcomma, iXStart, iY, "imgcomma"); break;
			/*** CTRL ***/
			case '$': iWidth = ShowImage
				(imgdollar_sign, iXStart, iY, "imgdollar_sign"); break;
			case '=': iWidth = ShowImage
				(imgequals_sign, iXStart, iY, "imgequals_sign"); break;
			/*** ESC ***/
			case '!': iWidth = ShowImage
				(imgexclamation_mark, iXStart, iY, "imgexclamation_mark"); break;
			case '`': iWidth = ShowImage
				(imggrave_accent, iXStart, iY, "imggrave_accent"); break;
			case '<': iWidth = ShowImage
				(imgguillemet_left, iXStart, iY, "imgguillemet_left"); break;
			case '>': iWidth = ShowImage
				(imgguillemet_right, iXStart, iY, "imgguillemet_right"); break;
			case '-': iWidth = ShowImage
				(imghyphen_minus, iXStart, iY, "imghyphen_minus"); break;
			case '#': iWidth = ShowImage
				(imgnumber_sign, iXStart, iY, "imgnumber_sign"); break;
			case ')': iWidth = ShowImage
				(imgparentheses_close, iXStart, iY, "imgparentheses_close"); break;
			case '(': iWidth = ShowImage
				(imgparentheses_open, iXStart, iY, "imgparentheses_open"); break;
			case '%': iWidth = ShowImage
				(imgpercent_sign, iXStart, iY, "imgpercent_sign"); break;
			case '.': iWidth = ShowImage
				(imgperiod, iXStart, iY, "imgperiod"); break;
			case '+': iWidth = ShowImage
				(imgplus_sign, iXStart, iY, "imgplus_sign"); break;
			case '?': iWidth = ShowImage
				(imgquestion_mark, iXStart, iY, "imgquestion_mark"); break;
			case '"': iWidth = ShowImage
				(imgquotation_marks, iXStart, iY, "imgquotation_marks"); break;
			case ';': iWidth = ShowImage
				(imgsemicolon, iXStart, iY, "imgsemicolon"); break;
			/*** slashed_o ***/
			case '/': iWidth = ShowImage
				(imgslash, iXStart, iY, "imgslash"); break;
			case ' ': iWidth = ShowImage
				(imgspace, iXStart, iY, "imgspace"); break;
			case '~': iWidth = ShowImage
				(imgtilde, iXStart, iY, "imgtilde"); break;
			case '_': iWidth = ShowImage
				(imgunderscore, iXStart, iY, "imgunderscore"); break;
		}
		iXStart = iXStart + iWidth + 1;
	}
}
/*****************************************************************************/
int ClickedValue (int iRow, int iButton)
/*****************************************************************************/
{
	int iY;

	iY = 35 + (iRow * 33);

	switch (iButton)
	{
		case 1: if (InArea (432, iY, 16, 8) == 1) { return (1); } break;
		case 2: if (InArea (432, iY + 8, 16, 8) == 1) { return (1); } break;
		case 3: if (InArea (527, iY, 16, 8) == 1) { return (1); } break;
		case 4: if (InArea (527, iY + 8, 16, 8) == 1) { return (1); } break;
	}

	return (0);
}
/*****************************************************************************/
int ClickedByte (int iRow, int iButton)
/*****************************************************************************/
{
	int iY;

	iY = 35 + (iRow * 33);

	switch (iButton)
	{
		case 1: if (InArea (432, iY, 16, 16) == 1) { return (1); } break;
		case 2: if (InArea (527, iY, 16, 16) == 1) { return (1); } break;
	}

	return (0);
}
/*****************************************************************************/
void Toggle (byte *bVar)
/*****************************************************************************/
{
	if (*bVar == 0) { *bVar = 1; } else { *bVar = 0; }

	redraw_screen (0);
}
/*****************************************************************************/
